window.Gleam.initSearch([{"doc":"mungo","title":"mungo","content":"![mungo](https://raw.githubusercontent.com/massivefermion/mungo/main/logo.png)\n\n[![Package Version](https://img.shields.io/hexpm/v/mungo)](https://hex.pm/packages/mungo)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/mungo/)\n\n# mungo (formerly gleam_mongo)\n&gt; mungo: a felted fabric made from the shredded fibre of repurposed woollen cloth\n---\n\n\nA mongodb driver for gleam\n\n## Quick start\n\n```sh\ngleam shell # Run an Erlang shell\n```\n\n## Installation\n\n```sh\ngleam add mungo\n```\n\n## Roadmap\n\n- [x] support basic mongodb commands\n- [x] support aggregation\n- [x] support connection strings\n- [x] support authentication\n- [x] support mongodb cursors\n- [ ] support connection pooling\n- [ ] support tls\n- [ ] support clusters\n- [ ] support bulk operations\n- [ ] support transactions\n- [ ] support change streams\n- [ ] support other mongodb commands\n\n## Usage\n\n```gleam\nimport gleam/uri\nimport gleam/option\nimport mungo\nimport mungo/crud.{Sort, Upsert}\nimport mungo/aggregation.{\n  Let, add_fields, aggregate, match, pipelined_lookup, to_cursor, unwind,\n}\nimport bison/bson\n\npub fn main() {\n  let encoded_password = uri.percent_encode(&quot;strong password&quot;)\n  let assert Ok(db) =\n    mungo.connect(\n      &quot;mongodb://app-dev:&quot; &lt;&gt; encoded_password &lt;&gt; &quot;@localhost/app-db?authSource=admin&quot;,\n    )\n\n  let users =\n    db\n    |&gt; mungo.collection(&quot;users&quot;)\n\n  let _ =\n    users\n    |&gt; mungo.insert_many([\n      [\n        #(&quot;username&quot;, bson.Str(&quot;jmorrow&quot;)),\n        #(&quot;name&quot;, bson.Str(&quot;vincent freeman&quot;)),\n        #(&quot;email&quot;, bson.Str(&quot;jmorrow@gattaca.eu&quot;)),\n        #(&quot;age&quot;, bson.Int32(32)),\n      ],\n      [\n        #(&quot;username&quot;, bson.Str(&quot;real-jerome&quot;)),\n        #(&quot;name&quot;, bson.Str(&quot;jerome eugene morrow&quot;)),\n        #(&quot;email&quot;, bson.Str(&quot;real-jerome@running.at&quot;)),\n        #(&quot;age&quot;, bson.Int32(32)),\n      ],\n    ])\n\n  let _ =\n    users\n    |&gt; mungo.update_one(\n      [#(&quot;username&quot;, bson.Str(&quot;real-jerome&quot;))],\n      [\n        #(\n          &quot;$set&quot;,\n          bson.Document([\n            #(&quot;username&quot;, bson.Str(&quot;eugene&quot;)),\n            #(&quot;email&quot;, bson.Str(&quot;eugene@running.at &quot;)),\n          ]),\n        ),\n      ],\n      [Upsert],\n    )\n\n  let assert Ok(yahoo_cursor) =\n    users\n    |&gt; mungo.find_many(\n      [#(&quot;email&quot;, bson.Regex(#(&quot;yahoo&quot;, &quot;&quot;)))],\n      [Sort([#(&quot;username&quot;, bson.Int32(-1))])],\n    )\n  let _yahoo_users = mungo.to_list(yahoo_cursor)\n\n  let assert Ok(underage_lindsey_cursor) =\n    users\n    |&gt; aggregate([Let([#(&quot;minimum_age&quot;, bson.Int32(21))])])\n    |&gt; match([\n      #(\n        &quot;$expr&quot;,\n        bson.Document([\n          #(&quot;$lt&quot;, bson.Array([bson.Str(&quot;$age&quot;), bson.Str(&quot;$$minimum_age&quot;)])),\n        ]),\n      ),\n    ])\n    |&gt; add_fields([\n      #(\n        &quot;first_name&quot;,\n        bson.Document([\n          #(\n            &quot;$arrayElemAt&quot;,\n            bson.Array([\n              bson.Document([\n                #(&quot;$split&quot;, bson.Array([bson.Str(&quot;$name&quot;), bson.Str(&quot; &quot;)])),\n              ]),\n              bson.Int32(0),\n            ]),\n          ),\n        ]),\n      ),\n    ])\n    |&gt; match([#(&quot;first_name&quot;, bson.Str(&quot;lindsey&quot;))])\n    |&gt; pipelined_lookup(\n      from: &quot;profiles&quot;,\n      define: [#(&quot;user&quot;, bson.Str(&quot;$username&quot;))],\n      pipeline: [\n        [\n          #(\n            &quot;$match&quot;,\n            bson.Document([\n              #(\n                &quot;$expr&quot;,\n                bson.Document([\n                  #(\n                    &quot;$eq&quot;,\n                    bson.Array([bson.Str(&quot;$username&quot;), bson.Str(&quot;$$user&quot;)]),\n                  ),\n                ]),\n              ),\n            ]),\n          ),\n        ],\n      ],\n      alias: &quot;profile&quot;,\n    )\n    |&gt; unwind(&quot;$profile&quot;, False)\n    |&gt; to_cursor\n\n  let assert #(option.Some(_underage_lindsey), underage_lindsey_cursor) =\n    underage_lindsey_cursor\n    |&gt; mungo.next\n\n  let assert #(option.None, _) =\n    underage_lindsey_cursor\n    |&gt; mungo.next\n}\n```\n","url":"index.html"},{"doc":"mungo/utils","title":"MongoError","content":"pub type MongoError {\n  MongoError(code: Int, msg: String, source: bson.Value)\n}\n\nMongoError(code: Int, msg: String, source: bson.Value)\n\n\nSynonyms:\nmungo/utils.MongoError\nmungo/utils MongoError","url":"mungo/utils.html#MongoError"},{"doc":"mungo/utils","title":"default_error","content":"pub const default_error: MongoError = MongoError(\n  code: -16,\n  msg: &quot;&quot;,\n  source: bson.Null,\n)\n\nSynonyms:\nmungo/utils.default_error\nmungo/utils default_error","url":"mungo/utils.html#default_error"},{"doc":"mungo/utils","title":"mungo/utils","content":"","url":"mungo/utils.html"},{"doc":"mungo/crud","title":"FindOption","content":"pub type FindOption {\n  Skip(Int)\n  Limit(Int)\n  BatchSize(Int)\n  Sort(List(#(String, bson.Value)))\n  Projection(List(#(String, bson.Value)))\n}\n\nSkip(Int)\n\n\nLimit(Int)\n\n\nBatchSize(Int)\n\n\nSort(List(#(String, bson.Value)))\n\n\nProjection(List(#(String, bson.Value)))\n\n\nSynonyms:\nmungo/crud.FindOption\nmungo/crud FindOption","url":"mungo/crud.html#FindOption"},{"doc":"mungo/crud","title":"InsertResult","content":"pub type InsertResult {\n  InsertResult(inserted: Int, inserted_ids: List(bson.Value))\n}\n\nInsertResult(inserted: Int, inserted_ids: List(bson.Value))\n\n\nSynonyms:\nmungo/crud.InsertResult\nmungo/crud InsertResult","url":"mungo/crud.html#InsertResult"},{"doc":"mungo/crud","title":"UpdateOption","content":"pub type UpdateOption {\n  Upsert\n  ArrayFilters(List(List(#(String, bson.Value))))\n}\n\nUpsert\n\n\nArrayFilters(List(List(#(String, bson.Value))))\n\n\nSynonyms:\nmungo/crud.UpdateOption\nmungo/crud UpdateOption","url":"mungo/crud.html#UpdateOption"},{"doc":"mungo/crud","title":"UpdateResult","content":"pub type UpdateResult {\n  UpdateResult(matched: Int, modified: Int)\n  UpsertResult(matched: Int, upserted_id: bson.Value)\n}\n\nUpdateResult(matched: Int, modified: Int)\n\n\nUpsertResult(matched: Int, upserted_id: bson.Value)\n\n\nSynonyms:\nmungo/crud.UpdateResult\nmungo/crud UpdateResult","url":"mungo/crud.html#UpdateResult"},{"doc":"mungo/crud","title":"count","content":"pub fn count(collection: Collection, filter: List(\n    #(String, Value),\n  )) -&gt; Result(Int, MongoError)\n\nSynonyms:\nmungo/crud.count\nmungo/crud count","url":"mungo/crud.html#count"},{"doc":"mungo/crud","title":"count_all","content":"pub fn count_all(collection: Collection) -&gt; Result(\n  Int,\n  MongoError,\n)\n\nSynonyms:\nmungo/crud.count_all\nmungo/crud count_all","url":"mungo/crud.html#count_all"},{"doc":"mungo/crud","title":"delete_many","content":"pub fn delete_many(collection: Collection, filter: List(\n    #(String, Value),\n  )) -&gt; Result(Int, MongoError)\n\nSynonyms:\nmungo/crud.delete_many\nmungo/crud delete_many","url":"mungo/crud.html#delete_many"},{"doc":"mungo/crud","title":"delete_one","content":"pub fn delete_one(collection: Collection, filter: List(\n    #(String, Value),\n  )) -&gt; Result(Int, MongoError)\n\nSynonyms:\nmungo/crud.delete_one\nmungo/crud delete_one","url":"mungo/crud.html#delete_one"},{"doc":"mungo/crud","title":"find_all","content":"pub fn find_all(collection: Collection, options: List(FindOption)) -&gt; Result(\n  Cursor,\n  MongoError,\n)\n\nSynonyms:\nmungo/crud.find_all\nmungo/crud find_all","url":"mungo/crud.html#find_all"},{"doc":"mungo/crud","title":"find_by_id","content":"pub fn find_by_id(collection: Collection, id: String) -&gt; Result(\n  Option(Value),\n  MongoError,\n)\n\nSynonyms:\nmungo/crud.find_by_id\nmungo/crud find_by_id","url":"mungo/crud.html#find_by_id"},{"doc":"mungo/crud","title":"find_many","content":"pub fn find_many(collection: Collection, filter: List(\n    #(String, Value),\n  ), options: List(FindOption)) -&gt; Result(Cursor, MongoError)\n\nSynonyms:\nmungo/crud.find_many\nmungo/crud find_many","url":"mungo/crud.html#find_many"},{"doc":"mungo/crud","title":"find_one","content":"pub fn find_one(collection: Collection, filter: List(\n    #(String, Value),\n  ), projection: List(#(String, Value))) -&gt; Result(\n  Option(Value),\n  MongoError,\n)\n\nSynonyms:\nmungo/crud.find_one\nmungo/crud find_one","url":"mungo/crud.html#find_one"},{"doc":"mungo/crud","title":"insert_many","content":"pub fn insert_many(collection: Collection, docs: List(\n    List(#(String, Value)),\n  )) -&gt; Result(InsertResult, MongoError)\n\nSynonyms:\nmungo/crud.insert_many\nmungo/crud insert_many","url":"mungo/crud.html#insert_many"},{"doc":"mungo/crud","title":"insert_one","content":"pub fn insert_one(collection: Collection, doc: List(\n    #(String, Value),\n  )) -&gt; Result(Value, MongoError)\n\nSynonyms:\nmungo/crud.insert_one\nmungo/crud insert_one","url":"mungo/crud.html#insert_one"},{"doc":"mungo/crud","title":"update_many","content":"pub fn update_many(collection: Collection, filter: List(\n    #(String, Value),\n  ), change: List(#(String, Value)), options: List(UpdateOption)) -&gt; Result(\n  UpdateResult,\n  MongoError,\n)\n for more information, see [here](https://www.mongodb.com/docs/manual/reference/operator/update)\nSynonyms:\nmungo/crud.update_many\nmungo/crud update_many","url":"mungo/crud.html#update_many"},{"doc":"mungo/crud","title":"update_one","content":"pub fn update_one(collection: Collection, filter: List(\n    #(String, Value),\n  ), change: List(#(String, Value)), options: List(UpdateOption)) -&gt; Result(\n  UpdateResult,\n  MongoError,\n)\n for more information, see [here](https://www.mongodb.com/docs/manual/reference/operator/update)\nSynonyms:\nmungo/crud.update_one\nmungo/crud update_one","url":"mungo/crud.html#update_one"},{"doc":"mungo/crud","title":"mungo/crud","content":"","url":"mungo/crud.html"},{"doc":"mungo","title":"collection","content":"pub fn collection(db: Database, name: String) -&gt; Collection\n\nSynonyms:\nmungo.collection\nmungo collection","url":"mungo.html#collection"},{"doc":"mungo","title":"connect","content":"pub fn connect(uri: String) -&gt; Result(Database, Nil)\n\nSynonyms:\nmungo.connect\nmungo connect","url":"mungo.html#connect"},{"doc":"mungo","title":"count","content":"pub fn count(collection: Collection, filter: List(\n    #(String, Value),\n  )) -&gt; Result(Int, MongoError)\n\nSynonyms:\nmungo.count\nmungo count","url":"mungo.html#count"},{"doc":"mungo","title":"count_all","content":"pub fn count_all(collection: Collection) -&gt; Result(\n  Int,\n  MongoError,\n)\n\nSynonyms:\nmungo.count_all\nmungo count_all","url":"mungo.html#count_all"},{"doc":"mungo","title":"delete_many","content":"pub fn delete_many(collection: Collection, filter: List(\n    #(String, Value),\n  )) -&gt; Result(Int, MongoError)\n\nSynonyms:\nmungo.delete_many\nmungo delete_many","url":"mungo.html#delete_many"},{"doc":"mungo","title":"delete_one","content":"pub fn delete_one(collection: Collection, filter: List(\n    #(String, Value),\n  )) -&gt; Result(Int, MongoError)\n\nSynonyms:\nmungo.delete_one\nmungo delete_one","url":"mungo.html#delete_one"},{"doc":"mungo","title":"find_all","content":"pub fn find_all(collection: Collection, options: List(FindOption)) -&gt; Result(\n  Cursor,\n  MongoError,\n)\n\nSynonyms:\nmungo.find_all\nmungo find_all","url":"mungo.html#find_all"},{"doc":"mungo","title":"find_by_id","content":"pub fn find_by_id(collection: Collection, id: String) -&gt; Result(\n  Option(Value),\n  MongoError,\n)\n\nSynonyms:\nmungo.find_by_id\nmungo find_by_id","url":"mungo.html#find_by_id"},{"doc":"mungo","title":"find_many","content":"pub fn find_many(collection: Collection, filter: List(\n    #(String, Value),\n  ), options: List(FindOption)) -&gt; Result(Cursor, MongoError)\n\nSynonyms:\nmungo.find_many\nmungo find_many","url":"mungo.html#find_many"},{"doc":"mungo","title":"find_one","content":"pub fn find_one(collection: Collection, filter: List(\n    #(String, Value),\n  ), projection: List(#(String, Value))) -&gt; Result(\n  Option(Value),\n  MongoError,\n)\n\nSynonyms:\nmungo.find_one\nmungo find_one","url":"mungo.html#find_one"},{"doc":"mungo","title":"insert_many","content":"pub fn insert_many(collection: Collection, docs: List(\n    List(#(String, Value)),\n  )) -&gt; Result(InsertResult, MongoError)\n\nSynonyms:\nmungo.insert_many\nmungo insert_many","url":"mungo.html#insert_many"},{"doc":"mungo","title":"insert_one","content":"pub fn insert_one(collection: Collection, doc: List(\n    #(String, Value),\n  )) -&gt; Result(Value, MongoError)\n\nSynonyms:\nmungo.insert_one\nmungo insert_one","url":"mungo.html#insert_one"},{"doc":"mungo","title":"next","content":"pub fn next(cursor: Cursor) -&gt; #(Option(Value), Cursor)\n\nSynonyms:\nmungo.next\nmungo next","url":"mungo.html#next"},{"doc":"mungo","title":"to_list","content":"pub fn to_list(cursor: Cursor) -&gt; List(Value)\n\nSynonyms:\nmungo.to_list\nmungo to_list","url":"mungo.html#to_list"},{"doc":"mungo","title":"update_many","content":"pub fn update_many(collection: Collection, filter: List(\n    #(String, Value),\n  ), change: List(#(String, Value)), options: List(UpdateOption)) -&gt; Result(\n  UpdateResult,\n  MongoError,\n)\n\nSynonyms:\nmungo.update_many\nmungo update_many","url":"mungo.html#update_many"},{"doc":"mungo","title":"update_one","content":"pub fn update_one(collection: Collection, filter: List(\n    #(String, Value),\n  ), change: List(#(String, Value)), options: List(UpdateOption)) -&gt; Result(\n  UpdateResult,\n  MongoError,\n)\n\nSynonyms:\nmungo.update_one\nmungo update_one","url":"mungo.html#update_one"},{"doc":"mungo","title":"mungo","content":"","url":"mungo.html"},{"doc":"mungo/aggregation","title":"AggregateOption","content":"pub type AggregateOption {\n  BatchSize(Int)\n  Let(List(#(String, bson.Value)))\n}\n\nBatchSize(Int)\n\n\nLet(List(#(String, bson.Value)))\n\n\nSynonyms:\nmungo/aggregation.AggregateOption\nmungo/aggregation AggregateOption","url":"mungo/aggregation.html#AggregateOption"},{"doc":"mungo/aggregation","title":"Pipeline","content":"pub opaque type Pipeline\n\n\nSynonyms:\nmungo/aggregation.Pipeline\nmungo/aggregation Pipeline","url":"mungo/aggregation.html#Pipeline"},{"doc":"mungo/aggregation","title":"add_fields","content":"pub fn add_fields(pipeline: Pipeline, doc: List(#(String, Value))) -&gt; Pipeline\n\nSynonyms:\nmungo/aggregation.add_fields\nmungo/aggregation add_fields","url":"mungo/aggregation.html#add_fields"},{"doc":"mungo/aggregation","title":"aggregate","content":"pub fn aggregate(collection: Collection, options: List(\n    AggregateOption,\n  )) -&gt; Pipeline\n\nSynonyms:\nmungo/aggregation.aggregate\nmungo/aggregation aggregate","url":"mungo/aggregation.html#aggregate"},{"doc":"mungo/aggregation","title":"append_stage","content":"pub fn append_stage(pipeline: Pipeline, stage: #(String, Value)) -&gt; Pipeline\n\nSynonyms:\nmungo/aggregation.append_stage\nmungo/aggregation append_stage","url":"mungo/aggregation.html#append_stage"},{"doc":"mungo/aggregation","title":"group","content":"pub fn group(pipeline: Pipeline, doc: List(#(String, Value))) -&gt; Pipeline\n\nSynonyms:\nmungo/aggregation.group\nmungo/aggregation group","url":"mungo/aggregation.html#group"},{"doc":"mungo/aggregation","title":"limit","content":"pub fn limit(pipeline: Pipeline, count: Int) -&gt; Pipeline\n\nSynonyms:\nmungo/aggregation.limit\nmungo/aggregation limit","url":"mungo/aggregation.html#limit"},{"doc":"mungo/aggregation","title":"lookup","content":"pub fn lookup(pipeline: Pipeline, from from: String, local_field local_field: String, foreign_field foreign_field: String, alias alias: String) -&gt; Pipeline\n for more information, see [here](https://www.mongodb.com/docs/manual/reference/operator/aggregation/lookup)\nSynonyms:\nmungo/aggregation.lookup\nmungo/aggregation lookup","url":"mungo/aggregation.html#lookup"},{"doc":"mungo/aggregation","title":"match","content":"pub fn match(pipeline: Pipeline, doc: List(#(String, Value))) -&gt; Pipeline\n\nSynonyms:\nmungo/aggregation.match\nmungo/aggregation match","url":"mungo/aggregation.html#match"},{"doc":"mungo/aggregation","title":"pipelined_lookup","content":"pub fn pipelined_lookup(pipeline: Pipeline, from from: String, define definitions: List(\n    #(String, Value),\n  ), pipeline lookup_pipeline: List(List(#(String, Value))), alias alias: String) -&gt; Pipeline\n for more information, see [here](https://www.mongodb.com/docs/manual/reference/operator/aggregation/lookup)\nSynonyms:\nmungo/aggregation.pipelined_lookup\nmungo/aggregation pipelined_lookup","url":"mungo/aggregation.html#pipelined_lookup"},{"doc":"mungo/aggregation","title":"project","content":"pub fn project(pipeline: Pipeline, doc: List(#(String, Value))) -&gt; Pipeline\n\nSynonyms:\nmungo/aggregation.project\nmungo/aggregation project","url":"mungo/aggregation.html#project"},{"doc":"mungo/aggregation","title":"skip","content":"pub fn skip(pipeline: Pipeline, count: Int) -&gt; Pipeline\n\nSynonyms:\nmungo/aggregation.skip\nmungo/aggregation skip","url":"mungo/aggregation.html#skip"},{"doc":"mungo/aggregation","title":"sort","content":"pub fn sort(pipeline: Pipeline, doc: List(#(String, Value))) -&gt; Pipeline\n\nSynonyms:\nmungo/aggregation.sort\nmungo/aggregation sort","url":"mungo/aggregation.html#sort"},{"doc":"mungo/aggregation","title":"to_cursor","content":"pub fn to_cursor(pipeline: Pipeline) -&gt; Result(Cursor, MongoError)\n\nSynonyms:\nmungo/aggregation.to_cursor\nmungo/aggregation to_cursor","url":"mungo/aggregation.html#to_cursor"},{"doc":"mungo/aggregation","title":"unwind","content":"pub fn unwind(pipeline: Pipeline, path: String, preserve_null_and_empty_arrays: Bool) -&gt; Pipeline\n\nSynonyms:\nmungo/aggregation.unwind\nmungo/aggregation unwind","url":"mungo/aggregation.html#unwind"},{"doc":"mungo/aggregation","title":"unwind_with_index","content":"pub fn unwind_with_index(pipeline: Pipeline, path: String, index_field: String, preserve_null_and_empty_arrays: Bool) -&gt; Pipeline\n\nSynonyms:\nmungo/aggregation.unwind_with_index\nmungo/aggregation unwind_with_index","url":"mungo/aggregation.html#unwind_with_index"},{"doc":"mungo/aggregation","title":"mungo/aggregation","content":" for more information, see [here](https://www.mongodb.com/docs/manual/reference/operator/aggregation-pipeline)","url":"mungo/aggregation.html"}]);